@@ -1,7 +1,7 @@
 from datetime import datetime
-import logging
+from mage_ai.server.logger import Logger
 
-LOGGER = logging.getLogger(__name__)
+LOGGER = Logger().new_server_logger(__name__)
 
 
 def debug(text):
@@ -14,6 +14,7 @@ def error(text):
 
     LOGGER.error(f'[{now}][api.views] {text}')
 
+
 def info(text):
     now = datetime.utcnow().isoformat()
     LOGGER.info(f'[{now}][api.views] {text}')
