@@ -4,7 +4,9 @@ from mage_ai.api.policies.BasePolicy import BasePolicy
 
 
 class SessionPolicy(BasePolicy):
-    pass
+    @property
+    def entity(self):
+        return None, None
 
 
 SessionPolicy.allow_actions([
@@ -17,7 +19,7 @@ SessionPolicy.allow_actions([
     constants.UPDATE,
 ], scopes=[
     OauthScope.CLIENT_PRIVATE,
-])
+], condition=lambda policy: policy.has_at_least_viewer_role())
 
 SessionPolicy.allow_read([
     'expires',
@@ -25,12 +27,20 @@ SessionPolicy.allow_read([
     'user',
 ], scopes=[
     OauthScope.CLIENT_PUBLIC,
-    OauthScope.CLIENT_PRIVATE,
 ], on_action=[
     constants.CREATE,
-    constants.UPDATE,
 ])
 
+SessionPolicy.allow_read([
+    'expires',
+    'token',
+    'user',
+], scopes=[
+    OauthScope.CLIENT_PRIVATE,
+], on_action=[
+    constants.UPDATE,
+], condition=lambda policy: policy.has_at_least_viewer_role())
+
 SessionPolicy.allow_write([
     'email',
     'password',
