@@ -11,40 +11,36 @@ class UserPolicy(BasePolicy):
 
 UserPolicy.allow_actions([
     constants.CREATE,
+    constants.DELETE,
 ], scopes=[
-    OauthScope.CLIENT_PUBLIC,
-])
+    OauthScope.CLIENT_PRIVATE,
+], condition=lambda policy: policy.is_owner())
 
 UserPolicy.allow_actions([
     constants.DETAIL,
     constants.UPDATE,
 ], scopes=[
     OauthScope.CLIENT_PRIVATE,
-], condition=lambda policy: policy.is_current_user())
+], condition=lambda policy: policy.is_current_user() or policy.has_at_least_admin_role())
+
+UserPolicy.allow_actions([
+    constants.LIST,
+], scopes=[
+    OauthScope.CLIENT_PRIVATE,
+], condition=lambda policy: policy.has_at_least_admin_role())
 
 UserPolicy.allow_read(UserPresenter.default_attributes, scopes=[
     OauthScope.CLIENT_PRIVATE,
-], condition=lambda policy: policy.is_current_user())
+], condition=lambda policy: policy.is_current_user() or policy.has_at_least_admin_role())
 
 UserPolicy.allow_read(UserPresenter.default_attributes + [
     'token',
-], scopes=[
-    OauthScope.CLIENT_PUBLIC,
-], on_action=[
-    constants.CREATE,
-])
-
-UserPolicy.allow_write([
-    'avatar',
-    'email',
-    'first_name',
-    'last_name',
-    'username',
 ], scopes=[
     OauthScope.CLIENT_PRIVATE,
 ], on_action=[
-    constants.UPDATE,
-], condition=lambda policy: policy.is_current_user())
+    constants.CREATE,
+    constants.DELETE,
+], condition=lambda policy: policy.is_current_user() or policy.is_owner())
 
 UserPolicy.allow_write([
     'avatar',
@@ -53,9 +49,43 @@ UserPolicy.allow_write([
     'last_name',
     'password',
     'password_confirmation',
+    'password_current',
     'username',
 ], scopes=[
-    OauthScope.CLIENT_PUBLIC,
+    OauthScope.CLIENT_PRIVATE,
+], on_action=[
+    constants.UPDATE,
+], condition=lambda policy: policy.is_current_user() or policy.has_at_least_admin_role())
+
+UserPolicy.allow_write([
+    'roles',
+    'roles_new'
+], scopes=[
+    OauthScope.CLIENT_PRIVATE,
+], on_action=[
+    constants.UPDATE,
+], condition=lambda policy: policy.has_at_least_admin_role())
+
+UserPolicy.allow_write([
+    'owner',
+], scopes=[
+    OauthScope.CLIENT_PRIVATE,
+], on_action=[
+    constants.UPDATE,
+], condition=lambda policy: policy.is_owner())
+
+UserPolicy.allow_write([
+    'avatar',
+    'email',
+    'first_name',
+    'last_name',
+    'password',
+    'password_confirmation',
+    'roles',
+    'roles_new',
+    'username',
+], scopes=[
+    OauthScope.CLIENT_PRIVATE,
 ], on_action=[
     constants.CREATE,
-])
+], condition=lambda policy: policy.is_owner())
