@@ -13,6 +13,7 @@ KUBE_NAMESPACE = 'KUBE_NAMESPACE'
 KUBE_SERVICE_TYPE = 'KUBE_SERVICE_TYPE'
 KUBE_SERVICE_GCP_BACKEND_CONFIG = 'KUBE_SERVICE_GCP_BACKEND_CONFIG'
 KUBE_STORAGE_CLASS_NAME = 'KUBE_STORAGE_CLASS_NAME'
+KUBE_SERVICE_ACCOUNT_NAME = 'KUBE_SERVICE_ACCOUNT_NAME'
 CLOUD_SQL_CONNECTION_NAME = 'CLOUD_SQL_CONNECTION_NAME'
 CONNECTION_URL_SECRETS_NAME = 'CONNECTION_URL_SECRETS_NAME'
 DB_SECRETS_NAME = 'DB_SECRETS_NAME'
@@ -21,4 +22,4 @@ SERVICE_ACCOUNT_CREDENTIAL_FILE_PATH = 'SERVICE_ACCOUNT_CREDENTIAL_FILE_PATH'
 
 # K8s constants
 GCP_BACKEND_CONFIG_ANNOTATION = 'cloud.google.com/backend-config'
-LOAD_BALANCER_SERVICE_TYPE = 'LoadBalancer'
+NODE_PORT_SERVICE_TYPE = 'NodePort'
