@@ -1,6 +1,8 @@
 DESTINATIONS = [
+    dict(name='Amazon S3'),
     dict(name='BigQuery'),
     dict(name='Delta Lake S3'),
+    dict(name='Google Cloud Storage'),
     dict(
         module_name='MSSQL',
         name='Microsoft SQL Server',
@@ -8,6 +10,7 @@ DESTINATIONS = [
     ),
     dict(name='MySQL'),
     dict(name='PostgreSQL'),
+    dict(name='Redshift'),
     dict(name='Snowflake'),
     dict(name='Trino'),
 ]
