@@ -8,6 +8,7 @@ SQL_SOURCES = [
     dict(name='MySQL'),
     dict(name='PostgreSQL'),
     dict(name='Redshift'),
+    dict(name='Snowflake'),
 ]
 
 SOURCES = sorted([
@@ -17,9 +18,13 @@ SOURCES = sorted([
     dict(name='Azure Blob Storage'),
     dict(name='Chargebee'),
     dict(name='Commercetools'),
+    dict(name='Couchbase'),
+    dict(name='Datadog'),
+    dict(name='DynamoDB'),
     dict(name='Facebook Ads'),
     dict(name='Freshdesk'),
     dict(name='Front'),
+    dict(name='Google Ads'),
     dict(name='Google Analytics'),
     dict(name='Google Search Console'),
     dict(name='Google Sheets'),
@@ -29,8 +34,10 @@ SOURCES = sorted([
     dict(name='Monday'),
     dict(name='Outreach'),
     dict(name='Paystack'),
+    dict(name='Pipedrive'),
     dict(name='Postmark'),
     dict(name='Salesforce'),
     dict(name='Stripe'),
+    dict(name='Twitter Ads'),
     dict(name='Zendesk'),
 ] + SQL_SOURCES, key=lambda x: x['name'])
