@@ -5,6 +5,14 @@ execute_block method of mage_ai/data_preparation/models/block.py file.
 """
 
 
+def callback(function):
+    return function
+
+
+def custom(function):
+    return function
+
+
 def data_exporter(function):
     return function
 
@@ -13,6 +21,10 @@ def data_loader(function):
     return function
 
 
+def extension(function):
+    return function
+
+
 def sensor(function):
     return function
 
@@ -23,3 +35,11 @@ def transformer(function):
 
 def test(function):
     return function
+
+
+def on_success(function):
+    return function
+
+
+def on_failure(function):
+    return function
