@@ -1,9 +1,9 @@
+import os
+from typing import Dict
+
 from mage_ai.data_preparation.executors.block_executor import BlockExecutor
 from mage_ai.services.gcp.cloud_run import cloud_run
 from mage_ai.shared.hash import merge_dict
-from requests import get
-from typing import Dict
-import os
 
 
 class GcpCloudRunBlockExecutor(BlockExecutor):
@@ -23,21 +23,9 @@ class GcpCloudRunBlockExecutor(BlockExecutor):
         global_vars: Dict = None,
         **kwargs,
     ) -> None:
-        cmd = f'/app/run_app.sh '\
-              f'mage run {self.pipeline.repo_config.repo_path} {self.pipeline.uuid}'
-        options = [
-            f'--block_uuid {self.block_uuid}',
-            '--executor_type local_python',
-        ]
-        if self.execution_partition is not None:
-            options.append(f'--execution_partition {self.execution_partition}')
-        if block_run_id is not None:
-            ip = get('https://api.ipify.org').content.decode('utf8')
-            callback_url = f'http://{ip}:6789/api/block_runs/{block_run_id}'
-            options.append(f'--callback_url {callback_url}')
-        options_str = ' '.join(options)
+        cmd = self._run_commands(block_run_id, global_vars, **kwargs)
         cloud_run.run_job(
-            f'{cmd} {options_str}',
-            f'mage-data-prep-{block_run_id}',
+            ' '.join(cmd),
+            f'mage-data-prep-block-{block_run_id}',
             cloud_run_config=self.executor_config,
         )
