@@ -1,10 +1,9 @@
 from mage_ai.data_preparation.models.block.sql.utils.shared import (
     create_upstream_block_tables as create_upstream_block_tables_orig,
     interpolate_input,
-    should_cache_data_from_upstream,
 )
 from mage_ai.io.config import ConfigKey
-from typing import Dict
+from typing import Dict, List
 
 
 def create_upstream_block_tables(
@@ -14,6 +13,9 @@ def create_upstream_block_tables(
     configuration: Dict = None,
     execution_partition: str = None,
     cache_upstream_dbt_models: bool = False,
+    query: str = None,
+    dynamic_block_index: int = None,
+    dynamic_upstream_block_uuids: List[str] = None,
 ):
     create_upstream_block_tables_orig(
         loader,
@@ -28,12 +30,22 @@ def create_upstream_block_tables(
             ConfigKey.MYSQL_PORT,
         ],
         no_schema=True,
+        query=query,
+        dynamic_block_index=dynamic_block_index,
+        dynamic_upstream_block_uuids=dynamic_upstream_block_uuids,
     )
 
 
-def interpolate_input_data(block, query):
+def interpolate_input_data(
+    block,
+    query: str,
+    dynamic_block_index: int = None,
+    dynamic_upstream_block_uuids: List[str] = None,
+):
     return interpolate_input(
         block,
         query,
         lambda db, schema, tn: tn,
+        dynamic_block_index=dynamic_block_index,
+        dynamic_upstream_block_uuids=dynamic_upstream_block_uuids,
     )
