@@ -1,7 +1,7 @@
 from mage_ai.shared.array import find, unique_by
 from mage_ai.shared.hash import merge_dict
 from mage_ai.shared.utils import clean_name as clean_name_orig
-from typing import Any, Dict, List
+from typing import Any, Dict, List, Tuple
 import json
 import pandas as pd
 
@@ -14,38 +14,53 @@ def dynamic_block_uuid(
     block_uuid: str,
     metadata: Dict,
     index: int,
+    upstream_block_uuid: str = None,
 ) -> str:
     block_uuid_subname = metadata.get('block_uuid', index)
-    return f'{block_uuid}:{block_uuid_subname}'
+    uuid = f'{block_uuid}:{block_uuid_subname}'
+
+    if upstream_block_uuid:
+        parts = upstream_block_uuid.split(':')
+        if len(parts) >= 2:
+            upstream_indexes = ':'.join(parts[1:])
+            uuid = f'{uuid}:{upstream_indexes}'
+
+    return uuid
 
 
 def create_block_run_from_dynamic_child(
-    block: 'Block',
-    pipeline_run: 'PipelineRun',
+    block,
+    pipeline_run,
     block_metadata: Dict,
     index: int,
-) -> 'BlockRun':
+    upstream_block_uuid: str = None,
+):
     metadata = block_metadata.copy()
     metadata.update(dict(dynamic_block_index=index))
 
-    block_uuid = dynamic_block_uuid(block.uuid, metadata, index)
+    block_uuid = dynamic_block_uuid(
+        block.uuid,
+        metadata,
+        index,
+        upstream_block_uuid=upstream_block_uuid,
+    )
     block_run = pipeline_run.create_block_run(block_uuid, metrics=metadata)
 
     return block_run
 
 
-def create_block_runs_from_dynamic_block(
-    block: 'Block',
-    pipeline_run: 'PipelineRun',
+def dynamic_block_values_and_metadata(
+    block,
+    execution_partition: str = None,
     block_uuid: str = None,
-) -> List['BlockRun']:
+):
     block_uuid_original = block.uuid
     block_uuid = block_uuid_original if block_uuid is None else block_uuid
-    execution_partition = pipeline_run.execution_partition
 
     values = []
     block_metadata = []
-    for idx, output_name in enumerate(block.output_variables()):
+    output_vars = block.output_variables(execution_partition=execution_partition)
+    for idx, output_name in enumerate(output_vars):
         if idx == 0:
             values = block.pipeline.variable_manager.get_variable(
                 block.pipeline.uuid,
@@ -61,6 +76,28 @@ def create_block_runs_from_dynamic_block(
                 partition=execution_partition,
             )
 
+    return values, block_metadata
+
+
+def create_block_runs_from_dynamic_block(
+    block,
+    pipeline_run,
+    block_uuid: str = None,
+) -> List:
+    block_uuid_original = block.uuid
+    block_uuid = block_uuid_original if block_uuid is None else block_uuid
+    execution_partition = pipeline_run.execution_partition
+
+    values, block_metadata = dynamic_block_values_and_metadata(
+        block,
+        execution_partition,
+        block_uuid,
+    )
+
+    # SQL blocks will return a Pandas DataFrame
+    if type(values) is pd.DataFrame:
+        values = values.to_dict(orient='records')
+
     all_block_runs = []
     # Dynamic child blocks (aka created from a dynamic block)
     for downstream_block in block.downstream_blocks:
@@ -82,6 +119,7 @@ def create_block_runs_from_dynamic_block(
                     arr.append(block_uuid)
                 else:
                     arr.append(upstream_block.uuid)
+
             block_run = create_block_run_from_dynamic_child(
                 downstream_block,
                 pipeline_run,
@@ -89,6 +127,7 @@ def create_block_runs_from_dynamic_block(
                     dynamic_upstream_block_uuids=arr,
                 )),
                 idx,
+                upstream_block_uuid=block_uuid,
             )
             all_block_runs.append(block_run)
             dynamic_child_block_runs.append(block_run)
@@ -103,6 +142,17 @@ def create_block_runs_from_dynamic_block(
                 if find(lambda x: is_dynamic_block(x), b.upstream_blocks):
                     continue
 
+                """
+                If a descendant has an immediate upstream block that is different than
+                the current downstream_block that reduces output, skip this loop because
+                this descendant will be created by that other upstream block that reduces output
+                """
+                # if find(
+                #     lambda x: downstream_block.uuid != x.uuid and should_reduce_output(x),
+                #     b.upstream_blocks,
+                # ):
+                #     continue
+
                 arr = []
                 for upstream_block in b.upstream_blocks:
                     ancestors = get_all_ancestors(upstream_block)
@@ -110,9 +160,21 @@ def create_block_runs_from_dynamic_block(
                     # then have this block depend on a block UUID with the dynamic UUID suffix;
                     # e.g. block_uuid:index
                     if downstream_block.uuid in [a.uuid for a in ancestors]:
-                        arr.append(dynamic_block_uuid(upstream_block.uuid, metadata, idx))
+                        arr.append(dynamic_block_uuid(
+                            upstream_block.uuid,
+                            metadata,
+                            idx,
+                        ))
                     elif downstream_block.uuid == upstream_block.uuid:
                         arr.append(block_run.block_uuid)
+                    elif is_dynamic_block_child(upstream_block):
+                        # Needs to know that the ancestors are dynamic
+                        # or dynamic child without reduce
+                        arr.append(dynamic_block_uuid(
+                            upstream_block.uuid,
+                            metadata,
+                            idx,
+                        ))
                     else:
                         arr.append(upstream_block.uuid)
 
@@ -141,16 +203,48 @@ def create_block_runs_from_dynamic_block(
 
                 skip_creating_downstream = False
                 ancestors_uuids = [a.uuid for a in ancestors]
+
                 for dynamic_ancestor in unique_dynamic_ancestors:
                     if skip_creating_downstream:
-                        continue
+                        break
 
-                    down_uuids = [down.uuid for down in dynamic_ancestor.downstream_blocks]
-                    down_uuids_as_ancestors = [i for i in down_uuids if i in ancestors_uuids]
+                    """
+                    If any dynamic ancestors doesn't have reduce block as the descendants, skip
+                    creating this downstream block.
+                    """
+                    dynamic_ancestor_descendants = get_all_descendants(dynamic_ancestor)
+
+                    dynamic_ancestor_has_reduce_block = False
+                    for d in dynamic_ancestor_descendants:
+                        if d.uuid in ancestors_uuids:
+                            if should_reduce_output(d):
+                                dynamic_ancestor_has_reduce_block = True
+                                break
+                    if not dynamic_ancestor_has_reduce_block:
+                        skip_creating_downstream = True
+                        break
+
+                    down_uuids_as_ancestors = []
+                    for down in dynamic_ancestor.downstream_blocks:
+                        if down.uuid in ancestors_uuids and not should_reduce_output(down):
+                            down_uuids_as_ancestors.append(down.uuid)
                     skip_creating_downstream = len(down_uuids_as_ancestors) >= 2
 
                 # Only create downstream block runs if it doesn’t have dynamically created upstream
-                # blocks (aka dynamic child) that were created by a 2nd ancestor that is a dynamic block
+                # blocks (aka dynamic child) that were created by a 2nd ancestor that is a dynamic
+                # block
+                if skip_creating_downstream:
+                    continue
+
+                for upstream_block in b.upstream_blocks:
+                    if should_reduce_output(upstream_block) and \
+                            upstream_block.uuid != downstream_block.uuid:
+                        skip_creating_downstream = True
+                        break
+                """
+                If an upstream block needs to be reduced but it's not the same with the current
+                block that's being reduced, skip the downstream block creation.
+                """
                 if skip_creating_downstream:
                     continue
 
@@ -164,12 +258,13 @@ def create_block_runs_from_dynamic_block(
                 all_block_runs.append(pipeline_run.create_block_run(
                     b.uuid,
                     metrics=dict(dynamic_upstream_block_uuids=arr),
+                    skip_if_exists=True,
                 ))
 
     return all_block_runs
 
 
-def get_all_ancestors(block: 'Block') -> List['Block']:
+def get_all_ancestors(block) -> List:
     arr = get_leaf_nodes(block, 'upstream_blocks', include_all_nodes=True)
     return list(filter(
         lambda x: x.uuid != block.uuid,
@@ -177,7 +272,7 @@ def get_all_ancestors(block: 'Block') -> List['Block']:
     ))
 
 
-def get_all_descendants(block: 'Block') -> List['Block']:
+def get_all_descendants(block) -> List:
     arr = get_leaf_nodes(block, 'downstream_blocks', include_all_nodes=True)
     return list(filter(
         lambda x: x.uuid != block.uuid,
@@ -186,14 +281,14 @@ def get_all_descendants(block: 'Block') -> List['Block']:
 
 
 def get_leaf_nodes(
-    block: 'Block',
+    block,
     attribute_key: str,
     condition=None,
     include_all_nodes: bool = False,
-) -> List['Block']:
+) -> List:
     leafs = []
 
-    def _get_leaf_nodes(b: 'Block'):
+    def _get_leaf_nodes(b):
         if condition is None or condition(b):
             if b is not None:
                 arr = getattr(b, attribute_key)
@@ -208,16 +303,31 @@ def get_leaf_nodes(
     return leafs
 
 
-def is_dynamic_block(block: 'Block') -> bool:
+def is_dynamic_block(block) -> bool:
     return block.configuration and block.configuration.get('dynamic', False)
 
 
-def should_reduce_output(block: 'Block') -> bool:
+def should_reduce_output(block) -> bool:
     return block.configuration and block.configuration.get('reduce_output', False)
 
 
+def is_dynamic_block_child(block) -> bool:
+    dynamic_or_child = []
+
+    for upstream_block in block.upstream_blocks:
+        if is_dynamic_block(upstream_block) or is_dynamic_block_child(upstream_block):
+            dynamic_or_child.append(upstream_block)
+
+    if len(dynamic_or_child) == 0:
+        return False
+
+    dynamic_or_child_with_reduce = list(filter(lambda x: should_reduce_output(x), dynamic_or_child))
+
+    return len(block.upstream_blocks) > len(dynamic_or_child_with_reduce)
+
+
 def output_variables(
-    pipeline: 'Pipeline',
+    pipeline,
     block_uuid: str,
     execution_partition: str = None,
 ) -> List[str]:
@@ -264,7 +374,7 @@ def is_valid_print_variable(k, v, block_uuid):
 
 
 def input_variables(
-    pipeline: 'Pipeline',
+    pipeline,
     upstream_block_uuids: List[str],
     execution_partition: str = None,
 ) -> Dict[str, List[str]]:
@@ -280,17 +390,19 @@ def input_variables(
 
 
 def fetch_input_variables(
-    pipeline: 'Pipeline',
+    pipeline,
     upstream_block_uuids: List[str],
     input_args: List[Any],
     execution_partition: str = None,
     global_vars: Dict = None,
     dynamic_block_index: int = None,
     dynamic_upstream_block_uuids: List[str] = None,
-):
+) -> Tuple[List, List, List]:
     spark = (global_vars or dict()).get('spark')
     upstream_block_uuids_final = []
 
+    kwargs_vars = []
+
     input_vars = []
     if input_args is not None:
         input_vars = input_args
@@ -333,9 +445,22 @@ def fetch_input_variables(
                 if len(variable_values) >= 1:
                     arr = variable_values[0]
                     index_to_use = 0 if dynamic_block_index is None else dynamic_block_index
+
+                    # SQL blocks will return a Pandas DataFrame
+                    if type(arr) is pd.DataFrame:
+                        val = arr.iloc[index_to_use].to_dict()
+                    elif type(arr) is list and len(arr) >= 1 and index_to_use < len(arr):
+                        val = arr[index_to_use]
+
+                input_vars[idx] = val
+
+                # output_0 is the metadata for dynamic blocks
+                if len(variable_values) >= 2:
+                    arr = variable_values[1]
+                    index_to_use = 0 if dynamic_block_index is None else dynamic_block_index
                     if type(arr) is list and len(arr) >= 1 and index_to_use < len(arr):
                         val = arr[index_to_use]
-                input_vars[idx] = val
+                    kwargs_vars.append(val)
             elif not dynamic_upstream_block_uuids or not upstream_in_dynamic_upstream:
                 if type(variable_values) is list and len(variable_values) == 1:
                     final_val = variable_values[0]
@@ -366,12 +491,12 @@ def fetch_input_variables(
                 )
 
                 final_value = []
-                for key, variables in input_variables_by_uuid.items():
+                for upstream_block_uuid, variables in input_variables_by_uuid.items():
                     end_idx = 1 if upstream_is_dynamic else len(variables)
                     for var in variables[:end_idx]:
                         variable_values = pipeline.variable_manager.get_variable(
                             pipeline.uuid,
-                            key,
+                            upstream_block_uuid,
                             var,
                             partition=execution_partition,
                             spark=spark,
@@ -387,20 +512,41 @@ def fetch_input_variables(
                         else:
                             final_value.append(val)
 
-                if len(final_value) >= 1 and type(final_value[0]) is pd.DataFrame:
-                    final_value = pd.concat(final_value)
+                    # output_0 is the metadata for dynamic blocks
+                    if dynamic_block_index is not None and \
+                            upstream_is_dynamic and \
+                            len(variables) >= 2:
 
+                        var = variables[1]
+                        variable_values = pipeline.variable_manager.get_variable(
+                            pipeline.uuid,
+                            upstream_block_uuid,
+                            var,
+                            partition=execution_partition,
+                            spark=spark,
+                        )
+                        if dynamic_block_index < len(variable_values):
+                            val = variable_values[dynamic_block_index]
+                            kwargs_vars.append(val)
+
+                if len(final_value) >= 1 and all([type(v) is pd.DataFrame for v in final_value]):
+                    final_value = pd.concat(final_value)
 
                 if not should_reduce:
                     # Only get the 1st output of a dynamic block;
                     # the 2nd output is the dynamic child block’s metadata
                     if upstream_is_dynamic and dynamic_block_index is not None:
-                        final_value = final_value[dynamic_block_index]
+                        # SQL blocks will return a Pandas DataFrame
+                        if type(final_value) is pd.DataFrame:
+                            final_value = final_value.iloc[dynamic_block_index].to_dict()
+                        else:
+                            final_value = final_value[dynamic_block_index]
 
                     if type(final_value) is not pd.DataFrame and \
-                        len(final_value) == 1:
+                        type(final_value) is list and \
+                            len(final_value) == 1:
                         final_value = final_value[0]
 
                 input_vars[idx] = final_value
 
-    return input_vars, upstream_block_uuids_final
+    return input_vars, kwargs_vars, upstream_block_uuids_final
