@@ -15,7 +15,7 @@ def load_data_from_big_query(*args, **kwargs):
     Template for loading data from a BigQuery warehouse.
     Specify your configuration settings in 'io_config.yaml'.
 
-    Docs: https://docs.mage.ai/design/data-loading
+    Docs: https://docs.mage.ai/design/data-loading#bigquery
     """
     query = 'your_gbq_query'
     config_path = path.join(get_repo_path(), 'io_config.yaml')
