@@ -15,7 +15,7 @@ def load_from_google_cloud_storage(*args, **kwargs):
     Template for loading data from a Google Cloud Storage bucket.
     Specify your configuration settings in 'io_config.yaml'.
 
-    Docs: https://docs.mage.ai/design/data-loading
+    Docs: https://docs.mage.ai/design/data-loading#googlecloudstorage
     """
     config_path = path.join(get_repo_path(), 'io_config.yaml')
     config_profile = 'default'
