@@ -15,7 +15,7 @@ def load_data_from_postgres(*args, **kwargs):
     Template for loading data from a PostgreSQL database.
     Specify your configuration settings in 'io_config.yaml'.
 
-    Docs: https://docs.mage.ai/design/data-loading
+    Docs: https://docs.mage.ai/design/data-loading#postgresql
     """
     query = 'your PostgreSQL query'  # Specify your SQL query here
     config_path = path.join(get_repo_path(), 'io_config.yaml')
