@@ -8,6 +8,8 @@ import yaml
 class IOConfigKeys(str, Enum):
     AWS = 'AWS'
     BIGQUERY = 'BigQuery'
+    CLICKHOUSE = 'ClickHouse'
+    DRUID = 'Druid'
     FILE = 'File'
     POSTGRES = 'PostgreSQL'
     REDSHIFT = 'Redshift'
