@@ -12,9 +12,12 @@ def reload_all_repo_modules(content: str) -> None:
         if f'import {project_name}' not in line and f'from {project_name}' not in line:
             continue
 
-        regex = re.compile('from ([\w.]+) import|import ([\w.]+)')
+        regex = re.compile(r'from ([\w.]+) import|import ([\w.]+)')
         matches = re.findall(regex, line)
 
         if len(matches) >= 1:
             m1, m2 = matches[0]
-            importlib.reload(importlib.import_module(m1 or m2))
+            try:
+                importlib.reload(importlib.import_module(m1 or m2))
+            except Exception:
+                pass
